;| DoubleOgee.lsp [command name: DoubleOgee
To replace a two-arc-segment ogee-shaped LWPolyline with a double-ogee
  of four arc segments, two of them on each side "offset" from it at its mid-way
  vertex and tangential there, with outer vertices meeting source's endpoints.
Requires open-ended LWPolyline on unlocked Layer, with 3 vertices and 2
  arc segments only.  Asks again if something else [or nothing] is selected.
Remembers User-specified mid-way offset distance, independent of regular
  Offset command's, and offers as default on subsequent use.  Allows Through
  option, in which case asks User for Through point on one side only, and makes
  other side same distance from source at mid-way vertex as first-side result is.
Designed for "gentle" ogee shape, of relatively low degree of curvature, and
  with arc segments that meet tangentially at mid-way vertex with reversing
  curvature [i.e. to opposite sides from there].  Will "work" if segments of source
  Polyline are not tangential, but results may be unexpected.  Will also work if
  they curve to same side [not ogee-shaped].  More severe curvatures [e.g. closer
  to true "S" shape, or radius that is small relative to offset-mid-way distance], can
  also give unexpected results.
Draws on current Layer.  Deletes source Polyline.  For WCS use only.
Kent Cooper, 15 March 2019
|;
(defun C:DoubleOgee
  (/ *error* svnames svvals pl pldata osd cmde osm mid tan thrudist dst astart)
  (defun *error* (errmsg)
    (if (not (wcmatch errmsg "Function cancelled,quit / exit abort,console break"))
      (princ (strcat "\nError: " errmsg))
    ); if
    (mapcar 'setvar svnames svvals); reset System Variables
    (vla-endundomark doc)
    (princ)
  ); defun - *error*
  (vla-startundomark (setq doc (vla-get-activedocument (vlax-get-acad-object))))
  (setq ; System Variable saving/resetting without separate variables for each:
    svnames '(osmode cmdecho peditaccept blipmode)
    svvals (mapcar 'getvar svnames)
  ); setq
  (while
    (not
      (and
        (setq pl (car (entsel "\nTwo-arc-segment LWPolyline center-path to make Double-Ogee: ")))
        (setq pldata (entget pl))
        (= (cdr (assoc 70 (tblsearch "layer" (cdr (assoc 8 pldata))))) 0); Unlocked Layer
        (member '(0 . "LWPOLYLINE") pldata); entity type
        (member '(90 . 3) pldata); # of vertices
        (= (logand 1 (cdr (assoc 70 pldata))) 0); open only
        (/= (cdr (assoc 42 pldata)) 0.0); first one
        (/= (cdr (assoc 42 (cdr (member (assoc 42 pldata) pldata)))) 0.0); second
          ;; check only first 2 -- third bulge factor for non-existent segment past last
          ;;   vertex can be 0.0 [e.g. if 2 Arcs joined with PEDIT or JOIN]
      ); and
    ); not
    (prompt "\nNothing selected, locked Layer, or not two-arc-segment LWPolyline.")
  ); while
  (setq
    osd (getvar 'offsetdist)
    cmde (getvar 'cmdecho)
    osm (getvar 'osmode)
    mid (vlax-curve-getPointAtParam pl 1.0)
    tan (angle '(0 0 0) (vlax-curve-getFirstDeriv pl 1.0))
  ); setq
  (initget 6 "Through")
  (setq *DblOgeeOff ; global variable
    (cond
      ( (getdist
          (strcat
            "\nOffset distance at mid-way vertex or [Through] <"
            (cond
              ((or (numberp *DblOgeeOff) (> osd 0))
                ;; prior distance value if present, or other-than-Through regular Offset
                (rtos (cond (*DblOgeeOff) (osd))); string equivalent
              ); 1st condition
              ("Through");; prior value nothing or Through, regular Offset is Through [-1]
            ); cond
            ">: "
          ); strcat
        ); getdist
      ); User-input condition
      (*DblOgeeOff); Enter for prior-value default
      ((> osd 0) osd); Enter on first use w/ other-than-Through regular Offset
      ("Through"); no prior value & regular Offset is Through
    ); cond & *DblOgeeOff
  ); setq
  (if (= *DblOgeeOff "Through")
    (progn ; then -- need Offset to establish distance for other side
      (command "_.offset" "Through" (list pl mid) pause ""); User pick Through-point
      (setq thrudist (distance mid (vlax-curve-getPointAtParam (entlast) 1)))
      (entdel (entlast))
    ); progn
  ); if
  (setq dst (if (numberp *DblOgeeOff) *DblOgeeOff thrudist))
  (mapcar 'setvar svnames '(0 0 1 0)); set System Variables
  (command
    "_.arc" (setq astart (polar mid (+ tan (/ pi 2)) dst))
      "_e" (vlax-curve-getEndPoint pl) "_direction" (angtos tan (getvar 'aunits) 8)
        ;; [spelling more, e.g. "_end" or "_endpoint", is taken as Osnap call]
  ); command
  (setq plss (ssadd (entlast)))
  (command
    "_.arc" astart "_e" (vlax-curve-getStartPoint pl) "_direction" (angtos (+ tan pi) (getvar 'aunits) 8)
  ); command
  (ssadd (entlast) plss)
  (command
    "_.arc" (setq astart (polar mid (- tan (/ pi 2)) dst))
      "_e" (vlax-curve-getEndPoint pl) "_direction" (angtos tan (getvar 'aunits) 8)
  ); command
  (ssadd (entlast) plss)
  (command
    "_.arc" astart "_e" (vlax-curve-getStartPoint pl) "_direction" (angtos (+ tan pi) (getvar 'aunits) 8)
  ); command
  (ssadd (entlast) plss)
  (command "_.pedit" "_multiple" plss "" "_join" 0 "")
  (entdel pl)
  (mapcar 'setvar svnames svvals); reset
  (vla-endundomark doc)
  (princ)
); defun